<?php
/**
 * AqayePardakht Gateway for Easy Digital Downloads
 *
 * @author 				adt <info@AqayePardakht.ir>
 * @package 			ADT
 * @subpackage 			Toman
 */
if ( !defined( 'ABSPATH' ) )exit;

if ( !class_exists( 'EDD_Aqayepardakht_Gateway' ) ):

  class EDD_Aqayepardakht_Gateway {
    /**
     * Gateway keyname
     *
     * @var 				string
     */
    public $keyname;

    /**
     * Initialize gateway and hook
     *
     * @return 				void
     */
    public function __construct() {
      $this->keyname = 'aqayepardakht';

      add_filter( 'edd_payment_gateways', array( $this, 'edd_add' ) );
      add_action( $this->edd_format( 'edd_{key}_cc_form' ), array( $this, 'edd_cc_form' ) );
      add_action( $this->edd_format( 'edd_gateway_{key}' ), array( $this, 'edd_process' ) );
      add_action( $this->edd_format( 'edd_verify_{key}' ), array( $this, 'edd_verify' ) );
      add_filter( 'edd_settings_gateways', array( $this, 'edd_settings' ) );

      add_action( 'edd_payment_receipt_after', array( $this, 'edd_receipt' ) );

      add_action( 'init', array( $this, 'edd_listen' ) );
    }

    /**
     * Add gateway to list
     *
     * @param 				array $gateways Gateways array
     * @return 				array
     */
    public function edd_add( $gateways ) {
      global $edd_options;

      $gateways[ $this->keyname ] = array(
        'checkout_label' => isset( $edd_options[ 'aqayepardakht_label' ] ) ? $edd_options[ 'aqayepardakht_label' ] : 'پرداخت آنلاین آقای پرداخت',
        'admin_label' => 'آقای پرداخت'
      );

      return $gateways;
    }

    /**
     * CC Form
     * We don't need it anyway.
     *
     * @return 				bool
     */
    public function edd_cc_form() {
      return;
    }

    /**
     * Process the payment
     *
     * @param 				array $purchase_data
     * @return 				void
     */
    public function edd_process( $purchase_data ) {
      global $edd_options;
      @session_start();
      $payment = $this->edd_insert_payment( $purchase_data );

      if ( $payment ) {

        $api = ( isset( $edd_options[ $this->keyname . '_merchant' ] ) ? $edd_options[ $this->keyname . '_merchant' ] : '' );
        $description = 'خریدار : ' . $purchase_data[ 'user_info' ][ 'first_name' ] . ' ' . $purchase_data[ 'user_info' ][ 'last_name' ] . ' |  ایمیل خریدار : ' . $purchase_data[ 'user_info' ][ 'email' ];
        $redirect = add_query_arg( array( 'verify_' . $this->keyname => '1', 'paymentId' => $payment ), get_permalink( edd_get_option( 'success_page', false ) ) );


        $amount = intval( $purchase_data[ 'price' ] ) / 10;
        if ( edd_get_currency() == 'IRT' )
          $amount = $amount * 10;

        $datacreate = array(
          'pin' => $api,
          'amount' => $amount,
          'callback' => $redirect,
          'invoice_id' => $payment,
          'description' => $description
        );
        $argscreate = array(
          'body' => $datacreate,
          'timeout' => '45',
          'redirection' => '5',
          'httpsversion' => '1.0',
          'blocking' => true,
          'headers' => array(),
          'cookies' => array()
        );

        $create = wp_remote_post( 'https://panel.aqayepardakht.ir/api/create', $argscreate );

        $header = wp_remote_retrieve_headers( $create );
        $code_costumer = null;
        $code_costumer = wp_remote_retrieve_body( $create );

        if ( $code_costumer && !is_numeric( $code_costumer ) ) {
          edd_insert_payment_note( $payment, 'کد تراکنش آقای پرداخت: ' . $code_costumer );
          edd_update_payment_meta( $payment, 'aqayepardakht_transid', $code_costumer );
          $_SESSION[ 'ap_payment' ] = $payment;

          wp_redirect( "https://panel.aqayepardakht.ir/startpay/{$code_costumer}" );
        } else {
          edd_insert_payment_note( $payment, 'کد خطا: ' . $code_costumer );
          edd_insert_payment_note( $payment, 'علت خطا: ' . $this->error_reason( $code_costumer ) );
          edd_update_payment_status( $payment, 'failed' );

          edd_set_error( 'aqayepardakht_connect_error', 'در اتصال به درگاه مشکلی پیش آمد. علت: ' . $this->error_reason( $code_costumer ) );
          edd_send_back_to_checkout();
        }
      } else {
        edd_send_back_to_checkout( '?payment-mode=' . $purchase_data[ 'post_data' ][ 'edd-gateway' ] );
      }
    }


    /**
     * Verify the payment
     *
     * @return 				void
     */
    public function edd_verify() {
      global $edd_options;
      $payment = isset( $_GET[ 'paymentId' ] ) ? absint( $_GET[ 'paymentId' ] ) : null;
      if ( isset( $_POST[ 'transid' ] ) ) {
        $transid = sanitize_text_field( $_POST[ 'transid' ] );
        $cardnumber = sanitize_text_field( $_POST[ 'cardnumber' ] );
        $tracking_number = sanitize_text_field( $_POST[ 'tracking_number' ] );

        $amount = intval( edd_get_payment_amount( $payment ) ) / 10;
        if ( edd_get_currency() == 'IRT' )
          $amount = $amount * 10;

        $api = ( isset( $edd_options[ $this->keyname . '_merchant' ] ) ? $edd_options[ $this->keyname . '_merchant' ] : '' );

        $dataverify = array(
          'pin' => $api,
          'amount' => $amount,
          'transid' => $transid
        );

        $argsverify = array(
          'body' => $dataverify,
          'timeout' => '45',
          'redirection' => '5',
          'httpsversion' => '1.0',
          'blocking' => true,
          'headers' => array(),
          'cookies' => array()
        );

        $verify = wp_remote_post( 'https://panel.aqayepardakht.ir/api/verify', $argsverify );

        $header = wp_remote_retrieve_headers( $verify );
        $verifyCode = null;
        $verifyCode = wp_remote_retrieve_body( $verify );

        edd_empty_cart();

        if ( version_compare( EDD_VERSION, '2.1', '>=' ) )
          edd_set_payment_transaction_id( $payment, $transid );

        if ( $verifyCode == "1" ) {
          edd_insert_payment_note( $payment, 'کد تراکنش آقای پرداخت: ' . $transid );
          edd_insert_payment_note( $payment, 'شماره تراکنش آقای پرداخت: ' . $tracking_number );
          edd_insert_payment_note( $payment, 'شماره کارت پرداخت کننده: ' . $cardnumber );

          edd_update_payment_meta( $payment, 'aqayepardakht_refid', $transid );
          edd_update_payment_status( $payment, 'publish' );
          edd_send_to_success_page();
        } else {
          edd_update_payment_status( $payment, 'failed' );
          wp_redirect( get_permalink( $edd_options[ 'failure_page' ] ) );
          exit;
        }
      } else {
        edd_update_payment_status( $payment, 'failed' );
        wp_redirect( get_permalink( $edd_options[ 'failure_page' ] ) );

        exit;
      }
    }
    /**
     * Receipt field for payment
     *
     * @param 				object $payment
     * @return 				void
     */
    public function edd_receipt( $payment ) {
      $refid = edd_get_payment_meta( $payment, 'aqayepardakht_refid' );
      if ( $refid ) {
        echo '<tr class="aqayepardakht-ref-id-row bdt-field aqayepardakht-dev"><td><strong>شماره تراکنش آقای پرداخت:</strong></td><td>' . $refid . '</td></tr>';
      }
    }

    /**
     * Gateway settings
     *
     * @param 				array $settings
     * @return 				array
     */
    public function edd_settings( $settings ) {
      return array_merge( $settings, array(
        $this->keyname . '_header' => array(
          'id' => $this->keyname . '_header',
          'type' => 'header',
          'name' => '<strong>درگاه آقای پرداخت</strong> توسط <a href="https://AqayePardakht.ir">AqayePardakht.ir</a>'
        ),
        $this->keyname . '_merchant' => array(
          'id' => $this->keyname . '_merchant',
          'name' => 'پین درگاه',
          'type' => 'text',
          'size' => 'regular'
        ),
        $this->keyname . '_label' => array(
          'id' => $this->keyname . '_label',
          'name' => 'نام درگاه در صفحه پرداخت',
          'type' => 'text',
          'size' => 'regular',
          'std' => 'پرداخت آنلاین آقای پرداخت'
        )
      ) );
    }

    /**
     * Format a string, replaces {key} with $keyname
     *
     * @param 			string $string To format
     * @return 			string Formatted
     */
    private function edd_format( $string ) {
      return str_replace( '{key}', $this->keyname, $string );
    }

    /**
     * Inserts a payment into database
     *
     * @param 			array $purchase_data
     * @return 			int $payment
     */
    private function edd_insert_payment( $purchase_data ) {
      global $edd_options;

      $payment_data = array(
        'price' => $purchase_data[ 'price' ],
        'date' => $purchase_data[ 'date' ],
        'user_email' => $purchase_data[ 'user_email' ],
        'purchase_key' => $purchase_data[ 'purchase_key' ],
        'currency' => $edd_options[ 'currency' ],
        'downloads' => $purchase_data[ 'downloads' ],
        'user_info' => $purchase_data[ 'user_info' ],
        'cart_details' => $purchase_data[ 'cart_details' ],
        'status' => 'pending'
      );
      $payment = edd_insert_payment( $payment_data );

      return $payment;
    }

    /**
     * Listen to incoming queries
     *
     * @return 			void
     */
    public function edd_listen() {
      if ( isset( $_GET[ 'verify_' . $this->keyname ] ) && $_GET[ 'verify_' . $this->keyname ] ) {
        do_action( 'edd_verify_' . $this->keyname );
      }
    }

    public function error_reason( $error_id ) {
      $message = 'خطای ناشناخته';

      switch ( $error_id ) {
        case '-1':
          $message = 'amount نمی تواند خالی باشد';
          break;
        case '-2':
          $message = 'کد پین درگاه نمی تواند خالی باشد';
          break;
        case '-3':
          $message = 'callback نمی تواند خالی باشد';
          break;
        case '-4':
          $message = 'amount باید عددی باشد';
          break;
        case '-5':
          $message = 'amount باید بزرگتر از ۱۰۰ باشد';
          break;
        case '-6':
          $message = 'کد پین درگاه اشتباه هست';
          break;
        case '-7':
          $message = 'ایپی سرور با ایپی درگاه مطابقت ندارد';
          break;
        case '-8':
          $message = 'transid نمی تواند خالی باشد';
          break;
        case '-9':
          $message = 'تراکنش مورد نظر وجود ندارد';
          break;
        case '-10':
          $message = 'کد پین درگاه با درگاه تراکنش مطابقت ندارد';
          break;
        case '-11':
          $message = 'مبلغ با مبلغ تراکنش مطابقت ندارد';
          break;
        case '-12':
          $message = 'بانک وارد شده اشتباه میباشد';
          break;
        case '-13':
          $message = 'درگاه غیر فعال است';
          break;
        case '-14':
          $message = 'درگاه برروی سایت دیگری درحال استفاده است';
          break;
      }

      return $message;
    }
  }

endif;

new EDD_Aqayepardakht_Gateway;